; Video output routine for OSI440 
; Receives characters from ACIA @FC00 and sends them to minimal screen handler


STORE       =$F0	;zero page storage location
VIDSRC      =STORE   ;2 .WORD $D040
VIDDST      =STORE+2 ;2 .WORD $D000
VIDOFFSET   =STORE+4 ;1 .BYTE 0
VIDXLEN     =STORE+5 ;1 .BYTE 0 ;length of line in screen memory
VIDXMAX     =STORE+6 ;1 .BYTE 0 ;line X wrap position
YHOLD		=STORE+7 ;1 .BYTE 0 ;temp storage

ACIA	=$FC00
CR		=$0D
LF		=$0A

	
;OSI 65A GO command format at $0129
; $0129 - Y
; $012A - X
; $012B - A
; $012C - Proc Status  'P'
; $012D - stack pointer 'K'
; $012E - PC Hi
; $012F - PC Lo
 
;the following will be added by .EXE loader emitted by A65
;*=$0129
;.BYTE 0,0,0,0,$FD,$02,$00


	*=$0200	;program starts at $0200
	.EXE	*	;A65 emit 'GO' address

;erase video	
	LDA #$20
	LDY #$00
CLR1
	STA $D000,Y
	STA $D100,Y
	STA $D200,Y
	STA $D300,Y
	INY
	BNE CLR1
	JSR VIDINIT
	
KEYLOOP	
	LDA ACIA
	LSR A    
	BCC KEYLOOP 	;wait for ACIA
	LDA ACIA+1		;read char from ACIA
    JSR VIDOUT		;send to screen
    JMP KEYLOOP
	
	
VIDINIT				;init video display routine
	LDY #$00
	STY VIDOFFSET
	STY VIDDST
	STY VIDCR+1     ;cursor start

C1VID   ;C1VID  1K screen
	LDA #$20       ;video mem line len
	LDX #$D3       ;bottom line hi

VIDINI1 ;read video settings from ROM (actually our storage)
	
	STA VIDXLEN
	STA VIDSRC
	
	LDA STARTPOS    ;$65 for C1
	TAY
	AND #$0F
	STA VIDCR+1     ;cursor start offset
	STA VIDOFFSET
	SEC
	ADC LINELEN
	STA VIDXMAX     ;max width/x position
	TYA
	AND #$F0        ;calc cursor row
	TAY             ;bottom line low	
VIDINI2 
	STX VIDCLR+2    ;D3A0 or D780 ( X=bottom line hi)
	STX VIDFIX1+1   ;D3A0 or D780
	STY VIDCLR+1  
	STY VIDFIX1
	LDA #$D0
	STA VIDSRC+1    ;init to D040 or D020
	STA VIDDST+1    ;init to D000
	RTS
		
VIDOUT
	STY YHOLD 
	CMP #CR
	BEQ VIDCR
	PHA
	CMP #LF
	BEQ VIDLF
	LDY VIDOFFSET
VIDFIX1=*+1
	STA $D6C0,Y
	INY
	STY VIDOFFSET
	CPY VIDXMAX
	BNE VIDRETY
	JSR VIDCR
	JMP VIDLF
VIDRETY
	LDY YHOLD
	PLA
	RTS
VIDCR			;Carriage return
	LDA #0  	;(value updated to left margin on init)
	STA VIDOFFSET
	RTS

VIDLF	;Line feed
	LDY #$00
VIDLF1
	LDA (VIDSRC),Y  ;copy row by row
	STA (VIDDST),Y
	INY
	CPY VIDXLEN
	BNE VIDLF1
	TYA
	CLC
	ADC VIDSRC
	STA VIDSRC
	BCC VIDLF2
	INC VIDSRC+1
VIDLF2
	TYA
	CLC
	ADC VIDDST
	STA VIDDST
	BCC VIDLF3
	INC VIDDST+1
VIDLF3  

	LDA VIDDST+1
	CMP VIDCLR+2
	BNE VIDLF1-2
	LDA VIDDST
	CMP VIDCLR+1
	BNE VIDLF1-2

	LDA #$D0
	STA VIDSRC+1
	STA VIDDST+1
	LDA #$00
	STA VIDDST
	LDA VIDXLEN
	STA VIDSRC

	LDA #' '
VIDCLR
	STA $D6C0,Y     ; Clear last line (address updated by init)
	INY
	CPY VIDXLEN
	BNE VIDCLR
	INY
	JMP VIDRETY
	
FFE0DATA=*  ;duplicates $FFE0 ROM values used in C1P & Other OSI BASIC in ROM systems
            ;change address to $FFE0 when using OSI ROM
STARTPOS=FFE0DATA
LINELEN=FFE0DATA+1
VIDSIZE=FFE0DATA+2
	.BYTE $85
	.BYTE $17
	.BYTE $00
	
	; $FFE0 - cursor start & bottom row
	; $FFE1 - line len-1   (wrap position)
	; $FFE2 - 00 = 1K vid, 01=2K vid otherwise serial or other ROM
	; FFE0 65, 17, 00 (c1 ROM, cegmon)
	; FFE0 40, 3F, 01 (c2 ROM)
	; FFE0 4D, 2F, 01 (C1E cegmon 64x32)
	; FFE0 CC, 2F, 00 (C1E cegmon 64x16)
	